/* gpsk31 - PSK31 for Linux with a GTK+ Interface
 * Copyright (C) 2000 Luc Langehegermann, LX2GT
 * Copyright (C) 2005 Joop Stakenborg, PG4I
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */


 /*
  * menu.C Functions for the menus
  */


#include <gtk/gtk.h>
#include <stdio.h>
#include "main_screen.h"
#include "callback.h"
#include "call_dialog.h"
#include "misc.h"
#include "globals.h"
#include "menu.h"
#include <unistd.h>
#include <stdlib.h>
#include "file.h"
#include "server.h"
#include "text.h"

GtkUIManager *ui_manager;
const char *trx_mode_names[] = {
        "BPSK31",
        "QPSK31"
};

#define BPSK31 0
#define QPSK31 1
int mode;

static void send_call ();
static void send_name ();
static void send_qth ();
static void send_rsts ();
static void send_rstr ();
static void send_frequency ();
static void send_mycall ();
static void send_myqth ();
static void send_myname ();

static void mode_qpsk_toggled ();
static void mode_afc_toggled ();
static void mode_net_toggled ();
static void mode_cwid_toggled ();

/* Normal items */
static GtkActionEntry entries[] =
{
  { "FileMenu", NULL, "_File" },
  { "SendMenu", NULL, "S_end" },
  { "ModeMenu", NULL, "_Mode" },
  { "WindowMenu", NULL, "_Window" },
  { "HelpMenu", NULL, "_Help" },

    {"Send", GTK_STOCK_OPEN, "Send File", "<control>S", "", G_CALLBACK(file_send_dialog)},
    {"Log", GTK_STOCK_NEW, "Log to _File", "<control>F", "", G_CALLBACK(save_log_dialog)},
    {"Close", GTK_STOCK_CLOSE, "Close Lo_gfile", "<control>G", "", G_CALLBACK(close_logfile)},
    {"Quit", GTK_STOCK_QUIT, "_Quit GPSK31", "<control>Q", "", G_CALLBACK(exit_dialog)},

    {"Call", NULL, "_Call", "<control>C", "", G_CALLBACK(send_call)},
    {"Name", NULL, "_Name", "<control>N", "", G_CALLBACK(send_name)},
    {"QTH", NULL, "QT_H", "<control>H", "", G_CALLBACK(send_qth)},
    {"RST-S", NULL, "RS_T-S", "<control>T", "", G_CALLBACK(send_rsts)},
    {"Frequency", NULL, "Fr_equency", "<control>E", "", G_CALLBACK(send_frequency)},
    {"Mycall", NULL, "MyCa_ll", "<control>L", "", G_CALLBACK(send_mycall)},
    {"Myname", NULL, "MyNa_me", "<control>M", "", G_CALLBACK(send_myname)},
    {"MyQTH", NULL, "M_yQTH", "<control>Y", "", G_CALLBACK(send_myqth)},
    {"RST-R", NULL, "RST-_R", "<control>R", "", G_CALLBACK(send_rstr)},

    {"Manual", GTK_STOCK_HELP, "Manual", "<control>I", "", G_CALLBACK(help_dialog)},
    {"ChangeLog", GTK_STOCK_HELP, "ChangeLog", "<control>O", "", G_CALLBACK(changelog_dialog)},
    {"About", GTK_STOCK_ABOUT, "About GPSK31", "<control><shift>H", "", G_CALLBACK(about_dialog)},

};


static GtkToggleActionEntry toggle_entries[] =
{
    {"QPSK", NULL, "_QPSK", "<alt>Q", "", G_CALLBACK(mode_qpsk_toggled)},
    {"CW_ID", NULL, "_CW-ID", "<alt>C", "", G_CALLBACK(mode_cwid_toggled)},
    {"AFC", NULL, "_AFC", "<alt>A", "", G_CALLBACK(mode_afc_toggled)},
    {"NET", NULL, "_NET", "<alt>N", "", G_CALLBACK(mode_net_toggled)},

    {"Details", NULL, "QSO Details", "<control>W", "", G_CALLBACK(toggle_call_dialog)},
};

static const char *ui_description =
"<ui>"
"  <menubar name='MainMenu'>"
"	 <menu action='FileMenu'>"
"		<menuitem action='Send'/>"
"		<separator name='sep1'/>"
"		<menuitem action='Log'/>"
"		<menuitem action='Close'/>"
"		<separator name='sep2'/>"
"		<menuitem action='Quit'/>"
"	 </menu>"
"	 <menu action='SendMenu'>"
"		<menuitem action='Call'/>"
"		<menuitem action='Name'/>"
"		<menuitem action='QTH'/>"
"		<menuitem action='RST-S'/>"
"		<menuitem action='Frequency'/>"
"		<separator name='sep3'/>"
"		<menuitem action='Mycall'/>"
"		<menuitem action='Myname'/>"
"		<menuitem action='MyQTH'/>"
"		<menuitem action='RST-R'/>"
"	 </menu>"
"	 <menu action='ModeMenu'>"
"		<menuitem action='QPSK'/>"
"		<menuitem action='CW_ID'/>"
"		<menuitem action='AFC'/>"
"		<menuitem action='NET'/>"
"	 </menu>"
"	 <menu action='WindowMenu'>"
"		<menuitem action='Details'/>"
"	 </menu>"
"	 <menu action='HelpMenu'>"
"		<menuitem action='ChangeLog'/>"
"		<menuitem action='Manual'/>"
"		<menuitem action='About'/>"
"	 </menu>"
"  </menubar>"
"</ui>";

void
init_menu ()
{
	GtkActionGroup *action_group = gtk_action_group_new ("MenuActions");
	gtk_action_group_add_actions (action_group, entries,
		G_N_ELEMENTS (entries), main_screen.window);
	gtk_action_group_add_toggle_actions (action_group,
		toggle_entries, G_N_ELEMENTS (toggle_entries), main_screen.window);

	ui_manager = gtk_ui_manager_new ();
	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);

	GtkAccelGroup *accel_group = gtk_ui_manager_get_accel_group (ui_manager);
	gtk_window_add_accel_group (GTK_WINDOW (main_screen.window), accel_group);

	gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
	main_screen.menu = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
}

void about_dialog()
{
   static GtkWidget *window;
   static GtkWidget *label; 
   static GtkWidget *ok_button;

   window = gtk_dialog_new_with_buttons ("GPSK31 - About",
      GTK_WINDOW(main_screen.window), GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
   label = gtk_label_new ("\nGPSK31 - Linux GTK+ PSK31 Terminal\n" \
		          "Version "VERSION" - "DATE"\n\n" \
			  "DSP Routines by Hansi Reiser, DL9RDZ\n" \
			  "GTK+ GUI by Luc Langehegermann, LX2GT\n" \
			  "GTK+-2.0 porting and maintenance by Joop Stakenborg, PG4I\n" \
			  "Bug fixes and additions by Thomas Ries, HB9XAR\n" \
			  );
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(window)->vbox), label);
   gtk_widget_show (label);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
   ok_button = gtk_dialog_add_button (GTK_DIALOG(window), GTK_STOCK_OK, GTK_RESPONSE_OK);
   gtk_dialog_run (GTK_DIALOG(window));
   gtk_widget_destroy (window);
}

static void display_file (const gchar *title, const gchar *filename)
{
   static GtkWidget *window;
   static GtkWidget *ok_button, *tv, *scrolled;
   gchar *helpfile, buf[80], *b;
   FILE *in;
   GtkTextBuffer *buffer;
   GtkTextIter start, end;
   PangoFontDescription *font_desc;

   gchar *temp = g_strdup_printf ("GPSK31 - %s", title);
   window = gtk_dialog_new_with_buttons (temp,
      GTK_WINDOW(main_screen.window), GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
   g_free (temp);
   gtk_widget_set_size_request (window, 650, 400);
   scrolled = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(window)->vbox), scrolled);
   gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
   tv = gtk_text_view_new ();
   gtk_container_add (GTK_CONTAINER (scrolled), tv);
   helpfile = g_strdup_printf
     ("%s%s%s", DATADIR, G_DIR_SEPARATOR_S, filename);
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(tv));
   gtk_text_buffer_get_bounds (buffer, &start, &end);
   b = g_new0 (gchar, 100);
   in = fopen (helpfile, "r");
   if (in)
   {
      do
      {
        if (fgets (buf, 80, in) == NULL)
           break;
        else
        {
           b = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
           gtk_text_buffer_insert (buffer, &start, b, -1);
         }
       }
       while (!feof (in));
       fclose (in);
   }
   g_free (b);
   g_free (helpfile);
   font_desc = pango_font_description_from_string ("mono");
   gtk_widget_modify_font (tv, font_desc);
   pango_font_description_free (font_desc);
   gtk_widget_show_all (scrolled);

   ok_button = gtk_dialog_add_button (GTK_DIALOG(window), GTK_STOCK_OK, GTK_RESPONSE_OK);
   gtk_widget_grab_focus (scrolled);
   gtk_dialog_run (GTK_DIALOG(window));
   gtk_widget_destroy (window);

}

void changelog_dialog()
{
	display_file ("History of changes", "ChangeLog");
}

void help_dialog()
{
	display_file ("Hints", "README");
}

static void exit_destroy (GtkButton *button,  gpointer user_data)
{
	gtk_widget_destroy (GTK_WIDGET(user_data));
}

void exit_dialog()
{
   static GtkWidget    *window = NULL;
   static GtkWidget    *label;
   static GtkWidget    *yes_button;
   static GtkWidget    *no_button;

   window = gtk_dialog_new_with_buttons ("GPSK31 - Exit",
      GTK_WINDOW(main_screen.window), GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
   gtk_window_set_default_size (GTK_WINDOW(window), 300, 100);
   g_signal_connect (G_OBJECT (window), "destroy",
                          G_CALLBACK (gtk_widget_destroyed),
                          &window);
   label = gtk_label_new ("Do you really want to quit?"); 
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(window)->vbox), label);
   gtk_widget_show (label);

   yes_button = gtk_dialog_add_button (GTK_DIALOG(window), GTK_STOCK_OK, GTK_RESPONSE_OK);
   no_button = gtk_dialog_add_button (GTK_DIALOG(window), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);

   g_signal_connect (G_OBJECT (no_button), "clicked",
                       G_CALLBACK (exit_destroy), G_OBJECT (window));

   g_signal_connect (G_OBJECT (yes_button), "clicked",
			G_CALLBACK (gtk_main_quit), NULL);
   gtk_widget_show_all (window);
}

void warning_dialog (const gchar *str, gboolean exit_program)
{
   static GtkWidget     *window;
   static GtkWidget     *label;

   window = gtk_dialog_new_with_buttons ("GPSK31 - Warning",
      GTK_WINDOW(main_screen.window), GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
   gtk_window_set_default_size (GTK_WINDOW(window), 300, 100);
   gtk_dialog_add_button (GTK_DIALOG(window), GTK_STOCK_OK, GTK_RESPONSE_OK);
   label = gtk_label_new (NULL);
   gtk_label_set_markup (GTK_LABEL (label), str);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(window)->vbox), label);
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
   gtk_widget_show (label);

   gtk_dialog_run (GTK_DIALOG (window));
   if (exit_program)
     exit (0);
   else
     gtk_widget_destroy (window);
}

/* This unchecks the qso_details_menu */
void check_menu(const char *path, int state)
{
	GtkWidget *item =  gtk_ui_manager_get_widget (ui_manager, path);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(item), state);
}

/*
 * This functions sends the info entered in the qso details dialog
 */

void send_call ()
{
	commPutData (qsodata.call, -1);
	put_tx_window (qsodata.call);
}


void send_name ()
{
        commPutData (qsodata.name, -1);
	put_tx_window (qsodata.name);
}

void send_qth ()
{
        commPutData (qsodata.qth, -1);
	put_tx_window (qsodata.qth);
}

void send_rsts ()
{
        commPutData (qsodata.rst_s, -1);
	put_tx_window (qsodata.rst_s);
}

void send_rstr ()
{
        commPutData (qsodata.rst_r, -1);
	put_tx_window (qsodata.rst_r);
}

void send_frequency ()
{
        commPutData (qsodata.freq, -1);
	put_tx_window (qsodata.freq);
}

void send_mycall ()
{
        commPutData (ini_settings.call, -1);
	put_tx_window (ini_settings.call);
}

void send_myqth ()
{
        commPutData (ini_settings.qth, -1);
	put_tx_window (ini_settings.qth);
}

void send_myname ()
{
        commPutData (ini_settings.name, -1);
	put_tx_window (ini_settings.name);
}

const char *trx_get_mode_name(void)
{
  return trx_mode_names[mode];
}


void mode_qpsk_toggled ()
{
	if (GTK_CHECK_MENU_ITEM (gtk_ui_manager_get_widget (ui_manager, "/MainMenu/ModeMenu/QPSK"))->active)
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.mode_qpsk), TRUE);
		mode = QPSK31;
	}
	else
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.mode_qpsk), FALSE);
		mode = BPSK31;
	}
}
                                                        
void mode_afc_toggled ()
{
	if (GTK_CHECK_MENU_ITEM (gtk_ui_manager_get_widget (ui_manager, "/MainMenu/ModeMenu/AFC"))->active)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.afc), TRUE);
	else    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.afc), FALSE);
}

void mode_net_toggled ()
{
        if (GTK_CHECK_MENU_ITEM (gtk_ui_manager_get_widget (ui_manager, "/MainMenu/ModeMenu/NET"))->active)
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.net), TRUE);
        else    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.net), FALSE);
}

void mode_cwid_toggled ()
{
	if (GTK_CHECK_MENU_ITEM (gtk_ui_manager_get_widget (ui_manager, "/MainMenu/ModeMenu/CW_ID"))->active)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.cwid), TRUE);
	else 
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (main_screen.cwid), FALSE);
}
